package Classes 
{
  import flash.geom.*;
  import flash.display.*;
  import flash.utils.*;
  import flash.events.*;
  import flash.net.*;
  import mx.preloaders.*; 
  import mx.events.*;
  import mx.controls.*;

  public class PreLoader extends Sprite implements IPreloaderDisplay
  {
    protected var _msecMinimumDuration:Number = 1000;   // even if the preloader is done, take this long to "finish"

    // Implementation variables, used to make everything work properly
    protected var _IsInitComplete:Boolean = false;
    protected var _timer:Timer;                 // we have a timer for animation
    protected var _bytesLoaded:uint = 0;
    protected var _bytesExpected:uint = 1;      // we start at 1 to avoid division by zero errors.
    protected var _fractionLoaded:Number = 0;   // 0-1
        
    public function PreLoader()
    {
      super();
    }
    
    private var screenBuffer:BitmapData = null;
        
    // This function is called whenever the state of the preloader changes.  Use the _fractionLoaded variable to draw your progress bar.
    virtual protected function draw():void
    {          
      screenBuffer.copyPixels(titleScreenBitmap,
                              new Rectangle(0, 0, 480, 480),
                              new Point(0,0));  
                              
      screenBuffer.copyPixels(progressBitmap,
                              new Rectangle(21, 8, 286 * _fractionLoaded, 36),
                              new Point(96,305));                               
                              
      screenBuffer.copyPixels(progressBitmap,
                              new Rectangle(0, 50, 330, 70),
                              new Point(75,290)); 
                              
      graphics.clear();    
      graphics.beginBitmapFill(screenBuffer, null, false, false);
      graphics.drawRect(0, 0, 480, 480);
      graphics.endFill();                                
    }
        
    [Embed(source="../Images/TitleScreen.jpg")] private var titleScreenImg:Class;
    private var titleScreenBitmap:BitmapData = null;     
    
    [Embed(source="../Images/ProgressBar.png")] private var progressBarImg:Class;
    private var progressBitmap:BitmapData = null;  
    
    ////
    //// IPreloaderDisplay interface elements
    ////   check out the docs on IPreloaderDisplay to see more details.
        
    // This function is called when the PreloaderDisplayBase has been created and is ready for action.
    virtual public function initialize():void
    {
      screenBuffer = new BitmapData(480, 480, false, 0x00000000);    
    
      titleScreenBitmap = new BitmapData(480,480,true);
      titleScreenBitmap.draw(new titleScreenImg()); 
      
      progressBitmap = new BitmapData(330,120,true,0x00000000);
      progressBitmap.draw(new progressBarImg()); 
            
      _timer = new Timer(1);
      _timer.addEventListener(TimerEvent.TIMER, timerHandler);
      _timer.start();
    }
            
    protected var _preloader:Sprite;

    virtual public function set preloader(value:Sprite):void
    {
      _preloader = value;
        
      value.addEventListener(ProgressEvent.PROGRESS, progressHandler);    
      value.addEventListener(Event.COMPLETE, completeHandler);
                       
      value.addEventListener(FlexEvent.INIT_PROGRESS, initProgressHandler);
      value.addEventListener(FlexEvent.INIT_COMPLETE, initCompleteHandler);
    }

    virtual public function set backgroundAlpha(alpha:Number):void{}
    virtual public function get backgroundAlpha():Number { return 1; }
        
    protected var _backgroundColor:uint = 0xffffffff;
    virtual public function set backgroundColor(color:uint):void { _backgroundColor = color; }
    virtual public function get backgroundColor():uint { return _backgroundColor; }
        
    virtual public function set backgroundImage(image:Object):void {}
    virtual public function get backgroundImage():Object { return null; }
        
    virtual public function set backgroundSize(size:String):void {}
    virtual public function get backgroundSize():String { return "auto"; }
        
    protected var _stageHeight:Number = 480;
    virtual public function set stageHeight(height:Number):void { _stageHeight = height; }
    virtual public function get stageHeight():Number { return _stageHeight; }

    protected var _stageWidth:Number = 480;
    virtual public function set stageWidth(width:Number):void { _stageWidth = width; }
    virtual public function get stageWidth():Number { return _stageWidth; }

    virtual protected function progressHandler(event:ProgressEvent):void
    {
      _bytesLoaded = event.bytesLoaded;
      _bytesExpected = event.bytesTotal;
      _fractionLoaded = Number(_bytesLoaded) / Number(_bytesExpected);
            
      draw();
    }
        
    // Called when the download is complete, but initialization might not be done yet.  (I *think*)
    // Note that there are two phases- download, and init
    virtual protected function completeHandler(event:Event):void
    {
    }
      
    // Called from time to time as the initialization continues.        
    virtual protected function initProgressHandler(event:Event):void
    {
      draw();
    }
    
    // Called when both download and initialization are complete    
    virtual protected function initCompleteHandler(event:Event):void
    {
      _IsInitComplete = true;
    }

    // Called as often as possible
    virtual protected function timerHandler(event:Event):void
    {
      if (_IsInitComplete && getTimer() > _msecMinimumDuration)
      {    
        // We're done!
        _timer.stop();
        dispatchEvent(new Event(Event.COMPLETE));
      }
      else
        draw();
    }  
  }
}