package Classes 
{
  import flash.geom.*; 
  import flash.display.*; 

  public class Snake
  { 
    public static const NORTH:int       = 1;
    public static const EAST:int        = 2;
    public static const SOUTH:int       = 3;
    public static const WEST:int        = 4; 
  
    private var speed:int = 0; 
    private var eating:int = 0;  
    private var killed:Boolean = false;
    private var direction:int = NORTH;  
    private var newDirection:int = NORTH; 
    private var newDirectionLimiter:int = 0;   
    private var partPosition:Array = new Array();
    private var partDirection:Array = new Array();	
    private var partPhase:Array = new Array();	  
    private var deadPart:Array = new Array();   
  
    public function Snake(xPos:int,yPos:int,length:int,speed:int) 
    {      
      this.speed = speed;
    
      for (var i:int = 0; i < length; i++)
      {
        partPosition.push(new Point(xPos,yPos - i * speed)); 
        partDirection.push(direction);
        partPhase.push(i % 9);
      }
    }    
    
    public function kill():Boolean
    {
      if (!killed)
      {
        killed = true;
        
        for (var i:int = 0; i < partPosition.length; i++)       
          deadPart.push(0);
      }
      
      if ((deadPart.indexOf(0) == -1) && (deadPart.indexOf(1) == -1) && (deadPart.indexOf(2) == -1))
        return false;
        
      for (var j:int = 0; j <= 2; j++)                 
        if (deadPart.indexOf(j) != -1)
          while (true)
          {
            var index:int = (int)(Math.random() * (deadPart.length - 0.1));
            
            if (deadPart[index] == j)
            {
              deadPart[index] = deadPart[index] + 1;         
              return true;
            }
          }           
      
      return true;
    }  
    
    public function getPosition():Point
    {
      return partPosition[0];
    } 
    
    public function getPartPosition(part:int):Point
    {
      return partPosition[part];
    }         
    
    public function getDirection():int
    {
      return direction;
    }
    
    public function setDirection(direction:int):Boolean
    {
      if (newDirectionLimiter == 0)
      {
        newDirection = direction;
        return true;
      }
      
      return false;
    }
    
    public function getLength():int
    {
      return partPosition.length;
    }   
    
    public function getBounds(part:int):Rectangle
    {
      var pos:Point = partPosition[part];
      var direction:int = partDirection[part];
      var phase:int = partPhase[part];
      var rectangle:Rectangle = null;
               
      if (part == 0) //head
      {        
        phase = partPhase[1];         
        
        switch (direction)
        {
        case NORTH:
          rectangle = new Rectangle(pos.x - 6 + northHeadOffset[phase],pos.y + 12,13,16);
          break;          
        case EAST:
          rectangle = new Rectangle(pos.x - 4,pos.y + 6 - eastHeadOffset[phase],16,13); 
          break;          
        case SOUTH:         
          rectangle = new Rectangle(pos.x - 6 + southHeadOffset[phase],pos.y + 4,13,16);
          break;          
        case WEST:
          rectangle = new Rectangle(pos.x - 12,pos.y + 6 - westHeadOffset[phase],16,13);
          break;                             
        }
      }
      else
      if (part == partPosition.length - 1) //tail       
        switch (direction)
        {
        case NORTH:         
          rectangle = new Rectangle(pos.x - 5 + northHeadOffset[phase],pos.y + 3,11,10);
          break;
        case EAST: 
          rectangle = new Rectangle(pos.x - 6,pos.y + 5 - eastHeadOffset[phase],10,11);
          break;
        case SOUTH: 
          rectangle = new Rectangle(pos.x - 5 + southHeadOffset[phase],pos.y + 7,11,10); 
          break;
        case WEST:
          rectangle = new Rectangle(pos.x - 5,pos.y + 5 - westHeadOffset[phase],10,11);
          break;
        }                          
      else  //body
        switch (direction)
        {
        case NORTH:         
          rectangle = new Rectangle(pos.x - 6 + northHeadOffset[phase],pos.y + 2,12,4);
          break;
        case EAST: 
          rectangle = new Rectangle(pos.x - 2,pos.y + 6 - eastHeadOffset[phase],4,12);
          break;
        case SOUTH: 
          rectangle = new Rectangle(pos.x - 6 + southHeadOffset[phase],pos.y + 2,12,4); 
          break;
        case WEST:
          rectangle = new Rectangle(pos.x - 2,pos.y + 6 - westHeadOffset[phase],4,12);
          break;
        }
        
      return rectangle;
    }         
    
    public function move():void
    {    
      partPosition.pop();
      partDirection.pop();
      partPhase.pop();
           
      var position:Point = partPosition[0];
      var phase:int = partPhase[0];    
         
      switch (direction)
      {
      case NORTH:
        if ((newDirection != direction) && ((phase == 0) || (phase == 4))) 
        {                
          direction = newDirection; 
          newDirectionLimiter = 2;         
          
          if (direction == EAST)
            partPosition.unshift(new Point(position.x + 7,position.y + 8));
          else
            partPosition.unshift(phase == 4 ? new Point(position.x - 5,position.y + 10) : new Point(position.x - 6,position.y + 8)); 
                       
          partPhase.unshift(0);  
        }
        else
        {
          partPosition.unshift(new Point(position.x,position.y + speed));             
          partPhase.unshift(((phase - 1) < 0 ? 8 : phase - 1)  % 9);
        }    
        break;
      case EAST:
        if ((newDirection != direction) && ((phase == 0) || (phase == 4)))     
        { 
          direction = newDirection;
          newDirectionLimiter = 2;  
          
          if (direction == NORTH)
            partPosition.unshift(phase == 4 ? new Point(position.x + 4,position.y + 8) : new Point(position.x + 4,position.y + 5));
          else
            partPosition.unshift(phase == 4 ? new Point(position.x + 5,position.y - 6) : new Point(position.x + 4,position.y - 8));        

          partPhase.unshift(0);  
        }
        else      
        {
          partPosition.unshift(new Point(position.x + speed,position.y));          
          partPhase.unshift((phase + 1) % 9); 
        }                  
        break;
      case SOUTH:
        if ((newDirection != direction) && ((phase == 0) || (phase == 4))) 
        {                
          direction = newDirection;  
          newDirectionLimiter = 2;          
          
          if (direction == EAST)
            partPosition.unshift(phase == 4 ? new Point(position.x + 6,position.y - 6) : new Point(position.x + 8,position.y - 6));
          else
            partPosition.unshift(phase == 4 ? new Point(position.x - 6,position.y - 4) : new Point(position.x - 4,position.y - 5)); 
                       
          partPhase.unshift(0);  
        }
        else
        {      
          partPosition.unshift(new Point(position.x,position.y - speed));     
          partPhase.unshift((phase + 1) % 9); 
        }        
        break;
      case WEST:
        if ((newDirection != direction) && ((phase == 0) || (phase == 4))) 
        { 
          direction = newDirection;
          newDirectionLimiter = 2;  
          
          if (direction == NORTH)
            partPosition.unshift(phase == 4 ? new Point(position.x - 9,position.y + 5) : new Point(position.x - 7,position.y + 5));
          else
            partPosition.unshift(new Point(position.x - 7,position.y - 8));       
          
          partPhase.unshift(0);  
        }
        else      
        {      
          partPosition.unshift(new Point(position.x - speed,position.y));      
          partPhase.unshift(((phase - 1) < 0 ? 8 : phase - 1)  % 9); 
        }   
        break;
      }   
      
      partDirection.unshift(direction);
      
      if (newDirectionLimiter > 0)
        newDirectionLimiter--;
        
      if (eating > 0)
        eating--;
    } 
    
    public function append():void
    { 
      var position:Point = partPosition[partPosition.length - 1];      
      var direction:int = partDirection[partPosition.length - 1];
      var phase:int = partPhase[partPosition.length - 1];
          
      switch (direction)
      {
      case NORTH:
        partPosition.push(new Point(position.x,position.y - speed));   
        partPhase.push((phase + 1) % 9);   
        break;
      case EAST:
        partPosition.push(new Point(position.x - speed,position.y));      
        partPhase.push(((phase - 1) < 0 ? 8 : phase - 1)  % 9);                
        break;
      case SOUTH:   
        partPosition.push(new Point(position.x,position.y + speed));   
        partPhase.push(((phase - 1) < 0 ? 8 : phase - 1)  % 9);    
        break;
      case WEST:     
        partPosition.push(new Point(position.x + speed,position.y));
        partPhase.push((phase + 1) % 9); 
        break;
      }
        
      partDirection.push(direction); 
      
      eating = 7;           
    }    
    
    [Embed(source="../Images/Snake.png")] private var snakeImg:Class;

    private var bitmap:BitmapData = null;
    
    private static var northHeadOffset:Array = new Array(0,2,3,3,1,-1,-3,-3,-2);
    private static var eastHeadOffset:Array = new Array(2,3,3,2,-1,-2,-3,-2,-1);
    private static var southHeadOffset:Array = new Array(2,3,3,2,-1,-2,-3,-2,-1);
    private static var westHeadOffset:Array = new Array(0,2,3,3,1,-1,-3,-3,-2);  
    
    public function draw(screenBuffer:BitmapData,visible:Rectangle):void
    {
      if (bitmap == null)
      {
        bitmap = new BitmapData(96,250,true,0x00000000);
        bitmap.draw(new snakeImg());
      }    
    
      for (var i:int = partPosition.length - 1; i >= 0; i--)
      {
        var deadPartOffset:int = 0;
      
        if (deadPart.length > 0) 
          if (deadPart[i] == 3)
            continue;
          else
            deadPartOffset = deadPart[i] * 83;
      
        var pos:Point = partPosition[i];
        var direction:int = partDirection[i];
        var phase:int = partPhase[i];
               
        if (i == 0) //head
        {        
          phase = partPhase[1];         
        
          switch (direction)
          {
          case NORTH:         
            if ((eating > 0) && (deadPartOffset == 0)) 
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(55,66,18,17) ,
                                      new Point(pos.x - 8 + northHeadOffset[phase],visible.y - pos.y - 13)); 
            else
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(38,51 + deadPartOffset,15,17),
                                      new Point(pos.x - 7 + northHeadOffset[phase],visible.y - pos.y - 13)); 
            break;
          case EAST:
            if ((eating > 0) && (deadPartOffset == 0)) 
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(21,20,17,18),
                                      new Point(pos.x - 4,visible.y - pos.y - 8 + eastHeadOffset[phase])); 
            else            
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(23,38 + deadPartOffset,17,15),
                                      new Point(pos.x - 4,visible.y - pos.y - 7 + eastHeadOffset[phase])); 
            break;
          case SOUTH: 
            if ((eating > 0) && (deadPartOffset == 0)) 
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(20,66,18,17),
                                      new Point(pos.x - 8 + northHeadOffset[phase],visible.y - pos.y - 4)); 
            else
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(38,22 + deadPartOffset,15,17),
                                      new Point(pos.x - 7 + southHeadOffset[phase],visible.y - pos.y - 4)); 
            break;
          case WEST:
            if ((eating > 0) && (deadPartOffset == 0)) 
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(54,20,17,18),
                                      new Point(pos.x - 13,visible.y - pos.y - 8 + westHeadOffset[phase])); 
            else          
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(52,38 + deadPartOffset,17,15),
                                      new Point(pos.x - 13,visible.y - pos.y - 7 + westHeadOffset[phase])); 
            break;
          }                    
        }
        else
        if (i == partPosition.length - 1) //tail
        {        
          switch (direction)
          {
          case NORTH:         
            screenBuffer.copyPixels(bitmap,
                                    new Rectangle(80,2 + deadPartOffset,11,10),
                                    new Point(pos.x - 5 + northHeadOffset[phase],visible.y - pos.y - 3)); 
            break;
          case EAST: 
            screenBuffer.copyPixels(bitmap,
                                    new Rectangle(80,24 + deadPartOffset,10,11),
                                    new Point(pos.x - 6,visible.y - pos.y - 5 + eastHeadOffset[phase]));
            break;
          case SOUTH: 
            screenBuffer.copyPixels(bitmap,
                                    new Rectangle(80,13 + deadPartOffset,11,10),
                                    new Point(pos.x - 5 + southHeadOffset[phase],visible.y - pos.y - 7)); 
            break;
          case WEST:
            screenBuffer.copyPixels(bitmap,
                                    new Rectangle(80,35 + deadPartOffset,10,11),
                                    new Point(pos.x - 5,visible.y - pos.y - 5 + westHeadOffset[phase]));
            break;
          }                    
        }        
        else  //body
        {
          if ((direction == EAST) || (direction == WEST))
            screenBuffer.copyPixels(bitmap,
                                    new Rectangle(18 + phase * 4,0 + deadPartOffset,4,19),
                                    new Point(pos.x - 2,visible.y - pos.y - 9));
          else          
            screenBuffer.copyPixels(bitmap,
                                    new Rectangle(0,18 + phase * 4 + deadPartOffset,19,4),
                                    new Point(pos.x - 9,visible.y - pos.y - 2));
        }
        
        var prevDirection:int = i < partPosition.length - 1 ? partDirection[i + 1] : direction;
        
        if (prevDirection != direction) 
          switch (prevDirection)
          {
          case NORTH:
            if (direction == EAST)
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(72,51 + deadPartOffset,10,11),
                                      new Point(pos.x - 12,visible.y - pos.y - 5));             
            else
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(82,51 + deadPartOffset,10,11),
                                      new Point(pos.x + 2,visible.y - pos.y - 3));             
            break;
          case EAST:
            if (direction == NORTH)
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(82,62 + deadPartOffset,10,11),
                                      new Point(pos.x - 2,visible.y - pos.y + 2));             
            else
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(82,51 + deadPartOffset,10,11),
                                      new Point(pos.x - 4,visible.y - pos.y - 12));            
            break;
          case SOUTH:
            if (direction == EAST)
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(72,62 + deadPartOffset,10,11),
                                      new Point(pos.x - 11,visible.y - pos.y - 5));             
            else
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(82,62 + deadPartOffset,10,11),
                                      new Point(pos.x + 2,visible.y - pos.y - 3));             
            break;
          case WEST:
            if (direction == NORTH)
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(72,62 + deadPartOffset,10,11),
                                      new Point(pos.x - 3,visible.y - pos.y + 2));             
            else
              screenBuffer.copyPixels(bitmap,
                                      new Rectangle(72,51 + deadPartOffset,10,11),
                                      new Point(pos.x - 5,visible.y - pos.y - 13));            
            break;
          }          
      } 
    }
  } 
}
