private static const GAME_FADE_IN:int        = 1;
private static const GAME_START:int          = 2;
private static const GAME_STARTED:int        = 3;
private static const GAME_PLAYED:int         = 4;
private static const GAME_KILLED:int         = 5;
private static const GAME_FAILED:int         = 6;
private static const GAME_FADE_OUT:int       = 7;
private static const GAME_CONTINUE:int       = 8;
private static const GAME_COMPLETED:int      = 9;
private static const GAME_DEACTIVATED:int    = 10;

private static const LEVELS:int              = 5;
private static const DIFFICULTIES:int        = 3;

private var gameState:int = GAME_FADE_IN;
private var newState:int = 0;
private var saveState:int = 0;
private var gameTime:Number = 0;

private var level:int = 0;
private var difficulty:int = 0;
private var scrollPos:int = 0;
private var score:int = 0;
private var hiscore:int = 0;
private var lives:int = 3;
private var liveGainedTimeout:Number = 0;
private var playedWithKeyboard:Boolean = false;

private var background:Background = null;

private var snake:Snake = null;
private var eggs:Array = null;
private var failEgg:Egg = null;
private var brokenEggs:Array = null;

private var butterFly:ButterFly = null;
private var birdCounter:int = 0;
private var bird:Bird = null;

private function New_Game():void
{  
  level = 1;
  difficulty = 1;
  score = 0;
  lives = 3;
  saveState = 0;
  playedWithKeyboard = false;
    
  Create_Level();

  ChangeState_Game(GAME_FADE_IN);
}

private function Restart_Game():void
{  
  lives--;
  
  if (lives == 0)
  { 
    if (score > hiscore)
    {
      hiscore = score;
      
      StoreData(HISCORE);
    }
  
    state = GAME_MENU;
    ChangeState_GameMenu(TITLE_SHOW);
  }  
  else
  {
    playedWithKeyboard = false;
    
    Create_Level();
    
    ChangeState_Game(GAME_FADE_IN);
  }    
}

private function Stage_Completed():void
{
  level++;

  if (level > LEVELS)
  {
    level = 1;
    difficulty++; 
  }
  
  if (difficulty > DIFFICULTIES)
  {
    if (score > hiscore)
    {
      hiscore = score;
      
      StoreData(HISCORE);
    }  
  
    state = GAME_MENU;
    ChangeState_GameMenu(TITLE_SHOW);  
  }
  else
  {
    playedWithKeyboard = false;
      
    Create_Level();

    ChangeState_Game(GAME_FADE_IN);  
  }
}

private function Create_Level():void
{  
  tipPosition = SCREEN_WIDTH;
  scrollPos = 0;
  liveGainedTimeout = 0;
  
  background = new Background(level); 
  
  if (difficulty == 1)
    snake = new Snake(SCREEN_WIDTH / 2,SCREEN_HEIGHT / 3,25,3);     
  else
    snake = new Snake(SCREEN_WIDTH / 2,SCREEN_HEIGHT / 3,20,4);
    
  //eggs
  var step:int = 0;
  var range:int = 0;  
  var prevX:int = 0; 
  
  switch (difficulty)
  {
  case 1:
    step = 100;
    range = 20;
    break;
  case 2:
    step = 80;
    range = 17;
    break;
  case 3:
    step = 60;
    range = 14;
    break;
  }
  
  eggs = new Array();

  for (var i:int = 300; i <= background.getHeight() - 160; i += step)
  {
    var x:int = (int)(Math.random() * (SCREEN_WIDTH - 30)) + 15;
    var y:int = i - step / 4 + Math.random() * step / 2;
    
    var egg:Egg = new Egg(x,y);
    var limit:int = 25;
    
    if ((prevX > 0) && (Math.abs(x - prevX) < limit))       //check horizontal distance between eggs
    {
      i -= step; 
      
      if (limit > 1)
        limit--;
      
      continue;
    }    
    
    if (background.checkCollision(egg.getBounds2(range)))   //check collision background
    {
      i -= step; 
      
      continue;
    }
    
    eggs.push(egg);
    
    prevX = x;
  }
  
  brokenEggs = new Array();
  
  CreateButterFly();  
  CreateBird();    
}

private function CreateButterFly():void
{
  if (level == 5)
    return;

  butterFly = new ButterFly((int)(Math.random() * (SCREEN_WIDTH - 100)) + 50,SCREEN_HEIGHT + scrollPos + 80 + int(Math.random() * 400));
  
  if (butterFly.getPosition().y > background.getHeight() - 200)
    butterFly = null;
}

private function CreateBird():void
{
  if (level == 5)
    return;
    
  if (birdCounter == 0)
    birdCounter = Math.random() * 200 + 100;
  else
    if (birdCounter > 0)
    {
      birdCounter--;
      
      if (birdCounter == 0)      
      {
        var angle:int = int(Math.random() * 23 + 0.5) * 15;
      
        var x:Number = Math.sin(angle / (180 / Math.PI)) * 300 + int(Math.random() * 120) - 60;
        var y:Number = Math.cos(angle / (180 / Math.PI)) * 300 + int(Math.random() * 120) - 60; 
      
        bird = new Bird(SCREEN_WIDTH / 2 - x,SCREEN_HEIGHT / 2 + scrollPos - y,angle);
      }
    }
}

private function ChangeState_Game(newState:int):void
{  
  gameState = newState;
  gameTime = 0;
  
  if (gameState == GAME_STARTED)
    scrollPos = 0;
    
  if ((gameState == GAME_CONTINUE) || (gameState == GAME_DEACTIVATED))
    playedWithKeyboard = false;
    
  if (((gameState == GAME_CONTINUE) || (gameState == GAME_DEACTIVATED)) && (saveState == GAME_START))
  {
    gameState = saveState;
    saveState = 0;
  }
    //gameTime = 1;    //don't draw stage sign
}

private function Draw_Game(ellapsedTime:Number):void
{  
  gameTime += ellapsedTime;    
    
  background.draw(screenBuffer,new Rectangle(0,SCREEN_HEIGHT + scrollPos,SCREEN_WIDTH,SCREEN_HEIGHT));  
  
  for (var i:int = 0; i < brokenEggs.length; i++)
    brokenEggs[i].draw(screenBuffer,new Rectangle(0,SCREEN_HEIGHT + scrollPos,SCREEN_WIDTH,SCREEN_HEIGHT)); 
  
  for (var j:int = 0; j < eggs.length; j++)
    eggs[j].draw(screenBuffer,new Rectangle(0,SCREEN_HEIGHT + scrollPos,SCREEN_WIDTH,SCREEN_HEIGHT));  
  
  if (snake != null)
    snake.draw(screenBuffer,new Rectangle(0,SCREEN_HEIGHT + scrollPos,SCREEN_WIDTH,SCREEN_HEIGHT)); 
    
  if (butterFly != null)
    butterFly.draw(screenBuffer,new Rectangle(0,SCREEN_HEIGHT + scrollPos,SCREEN_WIDTH,SCREEN_HEIGHT));     
    
  if (bird != null)
    bird .draw(screenBuffer,new Rectangle(0,SCREEN_HEIGHT + scrollPos,SCREEN_WIDTH,SCREEN_HEIGHT));
    
  Draw_ScoreLives();   
  
  switch (gameState)
  {
  case GAME_FADE_IN:  
    Draw_StageSign(gameTime,gameTime);
  
    Apply_FadeEffect(1 - gameTime); 
    
    if (gameTime >= 1)
      ChangeState_Game(GAME_START);       
    break;
  case GAME_START: 
    var fracTime:Number = gameTime - int(gameTime);
     
    Draw_StageSign(1,int(gameTime + 1) % 2 == 0 ? fracTime : 1 - fracTime);
    Draw_Tutorial(false); 
    
    if ((newState == GAME_STARTED) && 
        (int(gameTime + 1) % 2 == 0 ? fracTime > 0.55 : fracTime < 0.55))
    {
      ChangeState_Game(GAME_STARTED);     
      newState = 0;
    }
    break;   
  case GAME_STARTED: 
    Draw_StageSign(1 - gameTime,1 - gameTime); 
    Draw_Tutorial(true);     
    
    //if (gameTime >= 1)
      ChangeState_Game(GAME_PLAYED);       
    break;
  case GAME_PLAYED:
    if (gameTime < 1) 
      Draw_StageSign(1 - gameTime,1 - gameTime); 
      
    Draw_Tutorial(true);       
  
    for (var i2:int = 0; i2 < eggs.length; i2++)
      if (!eggs[i2].update())
        if (eggs.length == 1)
         ChangeState_Game(GAME_COMPLETED);          
        else
          eggs.splice(i2,1);          
  
    if (snake != null)
    {
      snake.move();
      
      var obj:Object = CheckCollision();
      
      if (obj is Snake)
        ChangeState_Game(GAME_KILLED);
        
      if (obj is Egg)
      {
        failEgg = Egg(obj);
      
        failEgg.blink();
        ChangeState_Game(GAME_FAILED); 
      }
    }
    
    if (butterFly != null)
    {
      if (butterFly.getPosition().y < scrollPos + SCREEN_HEIGHT)  
        butterFly.update(); 
      
      if (butterFly.getPosition().y < scrollPos)  
        butterFly = null;
    }

    if (butterFly == null)
      CreateButterFly();
      
    if (bird != null)
    {
      if (!bird.update())
        bird = null; 
    }
    else
      CreateBird();        
    
    liveGainedTimeout -= ellapsedTime;
    
    if (liveGainedTimeout < 0)
      liveGainedTimeout = 0;
    
    scrollPos++;
    break;    
  case GAME_KILLED:
    Draw_Tutorial(true);    
  
    if (!snake.kill())
      ChangeState_Game(GAME_FADE_OUT);   
    break;
  case GAME_FAILED:
    Draw_Tutorial(true);  
      
    if (!failEgg.blink())
      ChangeState_Game(GAME_FADE_OUT);   
    break;    
  case GAME_FADE_OUT:         
    Draw_Tutorial(true);    
  
    Apply_FadeEffect(gameTime); 
    
    if (gameTime >= 1)
      if (saveState == 0)
        Restart_Game();  
      else
      {
        state = GAME_MENU;
        ChangeState_GameMenu(TITLE_SHOW);      
      }    
    break;
  case GAME_CONTINUE: 
    var fracTime2:Number = gameTime - int(gameTime);
     
    Draw_StageSign(1,int(gameTime + 1) % 2 == 0 ? fracTime2 : 1 - fracTime2); 
    Draw_Tutorial(false); 
    
    if (gameTime < 1)    
      Apply_FadeEffect(1 - gameTime); 
    
    /*if (gameTime >= 1)
    {
      ChangeState_Game(saveState);       
      saveState = 0;
    }*/
    break; 
  case GAME_DEACTIVATED: 
    var fracTime3:Number = gameTime - int(gameTime);
     
    Draw_StageSign(1,int(gameTime + 1) % 2 == 0 ? fracTime3 : 1 - fracTime3);
    Draw_Tutorial(false); 
    break;    
  case GAME_COMPLETED:
    snake.move();
    
    if (butterFly != null)
      if (butterFly.getPosition().y < scrollPos + SCREEN_HEIGHT)  
        butterFly.update();  
        
    if (bird != null)
      if (!bird.update())
        bird = null; 
              
    Draw_Tutorial(true);            
  
    Apply_FadeEffect(gameTime / 2); 
    
    if (gameTime >= 2)
      Stage_Completed();
    break;
  }  
  
  if (queuedClick != null)
    MouseDown_Game(queuedClick);
    
  if (queuedKeyDown != null)
    KeyDown_Game(queuedKeyDown);
  
  SetGameCursor(); 
}

private function CheckCollision():Object
{
  var headBounds:Rectangle = snake.getBounds(0);
  
  //check egg and the bottom of the screen
  for (var i:int = 0; i < eggs.length; i++)
    if (eggs[i].active())
    {
      var eggBounds:Rectangle = eggs[i].getBounds(); 
  
      if (eggBounds.y - eggBounds.height <= scrollPos)
      {
        PlaySound(GAMEOVER_SOUND);       
        return eggs[i];         
      }

      //check snake and egg
      if (headBounds.intersects(eggBounds))
      {      
        PlaySound(EGGCOLLECTED_SOUND); 
        eggs[i].eat();
        brokenEggs.push(new BrokenEgg(eggs[i].getPosition().x,eggs[i].getPosition().y));
        snake.append();
        
        score += 10;
        
        if ((score % 500 == 0) && (lives < 6))
        {
          lives++;
          liveGainedTimeout = 4;
        }
      }
    }
  
  //check snake and the edge of the screen
  if ((headBounds.x <= 0) || (SCREEN_WIDTH - headBounds.x - headBounds.width <= 0) ||
      (headBounds.y - headBounds.height <= scrollPos) || (SCREEN_HEIGHT + scrollPos - headBounds.y <= 0))
  {
    PlaySound(GAMEOVER_SOUND);  
    snake.kill();
    return snake;     
  }
  
  //check snake and the background
  if (background.checkCollision(headBounds))
  {
    PlaySound(GAMEOVER_SOUND);  
    snake.kill();
    return snake;     
  }  
    
  //check snakes body
  for (var i2:int = 10; i2 < snake.getLength(); i2++)
  {
    var partBounds:Rectangle = snake.getBounds(i2); 
    
    if (headBounds.intersects(partBounds))
    {
      PlaySound(GAMEOVER_SOUND);    
      snake.kill();
      return snake;         
    }
  }
  
  return null;
}

[Embed(source="Images/ScoreLives.png")] private var scoreLivesImg:Class;

private var scoreLivesBitmap:BitmapData = null;
private static var scoreDigitPos:Array = new Array(123,139,151,165,179,195,209,224,238,253,270);

private function Draw_ScoreLives():void
{
  if (scoreLivesBitmap == null)
  {
    scoreLivesBitmap = new BitmapData(290,30,true,0x00000000);
    scoreLivesBitmap.draw(new scoreLivesImg());
  }
           
	//score
  screenBuffer.copyPixels(scoreLivesBitmap,
                          new Rectangle(37, 0, 80, 30),
                          new Point(4,0));
                          
	var scoreStr:String = score.toString();
	var xpos:int = 90;
                          
  for (var i:int = 0; i < 4; i++)  
	{
		var digit:int = 4 - i > scoreStr.length ? 0 : scoreStr.charCodeAt(i - (4 - scoreStr.length)) - 48;
		var digitPos:int = scoreDigitPos[digit];
		var digitWidth:int = scoreDigitPos[digit + 1] - digitPos;
			
    screenBuffer.copyPixels(scoreLivesBitmap,
                            new Rectangle(digitPos, 0, digitWidth, 30),
                            new Point(xpos,0));
                            
    xpos += 15;
	}
	
	//hiscore	
  screenBuffer.copyPixels(scoreLivesBitmap,
                          new Rectangle(0, 0, 117, 30),
                          new Point(SCREEN_WIDTH - 186,0));	
         
  xpos = SCREEN_WIDTH - 66;
	
	var hiscoreStr:String = hiscore.toString();
                          
  for (var i2:int = 0; i2 <  4; i2++)  
	{
		var digit2:int = 4 - i2 > hiscoreStr.length ? 0 : hiscoreStr.charCodeAt(i2 - (4 - hiscoreStr.length)) - 48;
		var digitPos2:int = scoreDigitPos[digit2];
		var digitWidth2:int = scoreDigitPos[digit2 + 1] - digitPos2;
			
    screenBuffer.copyPixels(scoreLivesBitmap,
                            new Rectangle(digitPos2, 0, digitWidth2, 30),
                            new Point(xpos,0));
                            
    xpos += 15;
	}	
	
	//lives
  xpos = SCREEN_WIDTH / 2 - 10 - lives * 10;	
	
  for (var i3:int = 0; i3 < lives; i3++)  
  {
    if (((gameState != GAME_KILLED) && (gameState != GAME_FAILED) && (liveGainedTimeout == 0)) || (i3 > 0) ||(gameTime - int(gameTime) < 0.5))
      screenBuffer.copyPixels(scoreLivesBitmap,
                              new Rectangle(271, 0, 20, 30),
                              new Point(xpos,0));    
  
    xpos += 20;
  }
}

[Embed(source="Images/StageLabel.png")] private var stageLabelImg:Class;
[Embed(source="Images/DifficultyLabel.png")] private var difficultyLabelImg:Class;
[Embed(source="Images/StageReady.png")] private var stageReadyImg:Class;

private var stageLabelBitmap:BitmapData = null;
private var difficultyLabelBitmap:BitmapData = null;
private var stageReadyBitmap:BitmapData = null;

private function Draw_StageSign(stageOpacity:Number,readyOpacity:Number):void
{
  if (stageLabelBitmap == null)
  {
    stageLabelBitmap = new BitmapData(350,160,true,0x00000000);
    stageLabelBitmap.draw(new stageLabelImg());
    
    difficultyLabelBitmap = new BitmapData(450,150,true,0x00000000);
    difficultyLabelBitmap.draw(new difficultyLabelImg());    
    
    stageReadyBitmap = new BitmapData(240,250,true,0x00000000);
    stageReadyBitmap.draw(new stageReadyImg());
  }
  
  if (stageOpacity > 0.15)   
  {
    var yPos:int = 0;
    
    if (stageOpacity < 0.25)
      yPos = 128;
    else
    if (stageOpacity < 0.45)
      yPos = 96;
    else
    if (stageOpacity < 0.45)
      yPos = 64;           
    else
    if (stageOpacity < 0.55)
      yPos = 32; 
                          
    screenBuffer.copyPixels(stageLabelBitmap,
                            new Rectangle(0, yPos, 100, 32),
                            new Point(SCREEN_WIDTH / 2 - 80,SCREEN_HEIGHT / 2 - 130));
                            
    screenBuffer.copyPixels(stageLabelBitmap,
                            new Rectangle(int(level / 10) * 24 + 102, yPos, 24, 32),
                            new Point(SCREEN_WIDTH / 2 + 30,SCREEN_HEIGHT / 2 - 130)); 
                                                        
    screenBuffer.copyPixels(stageLabelBitmap,
                            new Rectangle((level % 10) * 24 + 102, yPos, 24, 32),
                            new Point(SCREEN_WIDTH / 2 + 54,SCREEN_HEIGHT / 2 - 130)); 
                            
    switch (difficulty)
    {
    case 1:
      screenBuffer.copyPixels(difficultyLabelBitmap,
                              new Rectangle(0, yPos, 150, 32),
                              new Point(SCREEN_WIDTH / 2 - 78,SCREEN_HEIGHT / 2 - 90));                             
      break;
    case 2:
      screenBuffer.copyPixels(difficultyLabelBitmap,
                              new Rectangle(150, yPos, 137, 32),
                              new Point(SCREEN_WIDTH / 2 - 71,SCREEN_HEIGHT / 2 - 90));                             
      break;
    case 3:
      screenBuffer.copyPixels(difficultyLabelBitmap,
                              new Rectangle(287, yPos, 150, 32),
                              new Point(SCREEN_WIDTH / 2 - 77,SCREEN_HEIGHT / 2 - 90));                             
      break;
    }
  }
                                 
  if (readyOpacity > 0.15)   
  {
    var yPos2:int = 0;
    
    if (readyOpacity < 0.25)
      yPos2 = 200;
    else
    if (readyOpacity < 0.45)
      yPos2 = 150;
    else
    if (readyOpacity < 0.45)
      yPos2 = 100;           
    else
    if (readyOpacity < 0.55)
      yPos2 = 50;      
                  
    screenBuffer.copyPixels(stageReadyBitmap,
                            new Rectangle(0, yPos2, 240, 50),
                            new Point(SCREEN_WIDTH / 2 - 120,SCREEN_HEIGHT / 2 - 10));  
  }
}

[Embed(source="Images/Tutorial.png")] private var tutorialImg:Class;

private var lastTip:int = 0;
private var tipPosition:int = SCREEN_WIDTH;
private var tutorialBitmap:BitmapData = null;

private function Draw_Tutorial(hide:Boolean):void
{
  if ((hide) && (tipPosition == SCREEN_WIDTH)) 
    return;

  if (tutorialBitmap == null)
  {
    tutorialBitmap = new BitmapData(400,126,true,0x00000000);
    tutorialBitmap.draw(new tutorialImg());
  }
  
  screenBuffer.copyPixels(tutorialBitmap,
                          new Rectangle(0, lastTip * 18, 400, 18),
                          new Point(tipPosition,SCREEN_HEIGHT - 20)); 
                          
  tipPosition -= 2;
  
  if (hide)
    tipPosition -= 1;   
  
  if (tipPosition <= -400)
  {
    tipPosition = SCREEN_WIDTH;
    lastTip++;
    
    if (lastTip == 7)
      lastTip = 0;
  }
}

private function SetGameCursor():void 	
{  
  if (playedWithKeyboard)
  {
    SetMouseCursor(EMPTY_CURSOR,false);     
    return;
  }

  var mx:int = mouseX;
  var my:int = mouseY;
  
  my = SCREEN_HEIGHT + scrollPos - my;
  var position:Point = snake.getPosition();
  var direction:int = snake.getDirection(); 
  var cursorDirection:int = 0;
  
  if ((direction == Snake.NORTH) || (direction == Snake.SOUTH))
    if (mx - position.x > 0)  
      cursorDirection = Snake.EAST;      
    else
      cursorDirection = Snake.WEST; 
  else
    if (my - position.y > 0)
      cursorDirection = Snake.NORTH;      
    else
      cursorDirection = Snake.SOUTH; 
  
  SetMouseCursor(cursorDirection,gameState > GAME_PLAYED);  
}

private var queuedClick:MouseEvent = null;

private function MouseDown_Game(event:MouseEvent):void 	
{	
  var mx:int = event.localX;
  var my:int = event.localY;
  
  switch (gameState)
  {
  case GAME_FADE_IN:
  case GAME_START:  
    newState = GAME_STARTED;   
    break;
  case GAME_PLAYED:
    {
      my = SCREEN_HEIGHT + scrollPos - my;
      var position:Point = snake.getPosition();
      var direction:int = snake.getDirection();
      var directionChanged:Boolean = true; 
      
      if ((direction == Snake.NORTH) || (direction == Snake.SOUTH))
        if (mx - position.x > 0)  
          directionChanged = snake.setDirection(Snake.EAST);      
        else
          directionChanged = snake.setDirection(Snake.WEST); 
      else
        if (my - position.y > 0)
          directionChanged = snake.setDirection(Snake.NORTH);      
        else
          directionChanged = snake.setDirection(Snake.SOUTH); 
          
      if (!directionChanged)
        queuedClick = event;
      else
        queuedClick = null;
    }
    break; 
  case GAME_CONTINUE:
    ChangeState_Game(saveState);       
    saveState = 0;
    break;
  case GAME_DEACTIVATED:
    ChangeState_Game(saveState);       
    saveState = 0;
    break;    
  } 
}	
	
private function MouseUp_Game(event:MouseEvent):void 	
{	
  playedWithKeyboard = false;
}

private function MouseMoved_Game(event:MouseEvent):void 	
{
  playedWithKeyboard = false;
}

private function MouseWheelMoved_Game(event:MouseEvent):void 	
{
  /*switch (gameState)
  {
  case GAME_START:  
	  if (event.delta > 0)
      scrollPos += 48 * 2;
	  else
      scrollPos -= 48 * 2;
      
    if (scrollPos < 0)
      scrollPos = 0;
    break;   
  }*/ 
}

private var queuedKeyDown:KeyboardEvent = null;

private function KeyDown_Game(event:KeyboardEvent):void 
{	  
  switch (gameState)
  {
  case GAME_FADE_IN:
  case GAME_START: 
    if ((event.keyCode == Keyboard.UP) || (event.keyCode == Keyboard.DOWN) || (event.keyCode == Keyboard.LEFT) || (event.keyCode == Keyboard.RIGHT))
    {
      playedWithKeyboard = true;    
      newState = GAME_STARTED;   
    }
    break;    
  case GAME_PLAYED:
    if ((event.keyCode == Keyboard.UP) || (event.keyCode == Keyboard.DOWN) || (event.keyCode == Keyboard.LEFT) || (event.keyCode == Keyboard.RIGHT))
    {
      playedWithKeyboard = true;
          
      var direction:int = snake.getDirection();
      var directionChanged:Boolean = true;     
          
      switch (event.keyCode)
      {
      case Keyboard.UP:
        if ((direction == Snake.EAST) || (direction == Snake.WEST))
          directionChanged = snake.setDirection(Snake.NORTH);
        break;
      case Keyboard.DOWN:	
        if ((direction == Snake.EAST) || (direction == Snake.WEST))
          directionChanged = snake.setDirection(Snake.SOUTH);
        break; 
      case Keyboard.LEFT:	
        if ((direction == Snake.NORTH) || (direction == Snake.SOUTH))
          directionChanged = snake.setDirection(Snake.WEST);    
        break; 
      case Keyboard.RIGHT:
        if ((direction == Snake.NORTH) || (direction == Snake.SOUTH))
          directionChanged = snake.setDirection(Snake.EAST);       
        break;       
      } 
      
      if (!directionChanged)
        queuedKeyDown = event;
      else
        queuedKeyDown = null;
    }
    break;
  case GAME_CONTINUE:
  case GAME_DEACTIVATED:  
    if ((event.keyCode == Keyboard.UP) || (event.keyCode == Keyboard.DOWN) || (event.keyCode == Keyboard.LEFT) || (event.keyCode == Keyboard.RIGHT))
    {
      playedWithKeyboard = true;
      ChangeState_Game(saveState);       
      saveState = 0;
    }
    break;    
  }
}

private function KeyUp_Game(event:KeyboardEvent):void 	
{	
	if (event.keyCode == Keyboard.ESCAPE)
	{
    saveState = gameState;
    ChangeState_Game(GAME_FADE_OUT);    
	}
}