private static const SETTINGSOPTIONS_IN:int   = 1;
private static const SETTINGSOPTIONS_WAIT:int = 2;
private static const SETTINGSOPTIONS_OUT:int   = 3;

private var settingsMenuState:int = SETTINGSOPTIONS_IN;
private var settingsMenuTime:Number = 0;

private static const OPTION_SOUNDVOLUME:int  = 0;
private static const OPTION_MUSICVOLUME:int  = 1;
private static const OPTION_EXIT:int         = 2;

private function Draw_SettingsMenu(ellapsedTime:Number):void
{ 
  settingsMenuTime += ellapsedTime;
   
  switch (settingsMenuState)
  {
  case SETTINGSOPTIONS_IN:
    Draw_TitleScreen();
    Draw_SettingsOptions();
    
    if (settingsMenuTime >= 1)
      ChangeState_SettingsMenu(SETTINGSOPTIONS_WAIT); 
    break;   
  case SETTINGSOPTIONS_WAIT: 
    Draw_TitleScreen();
    Draw_SettingsOptions();
    break;   
  case SETTINGSOPTIONS_OUT:  
    Draw_TitleScreen();
    Draw_SettingsOptions();
    
    if (settingsMenuTime >= 1)
    {
      state = GAME_MENU;
      ChangeState_GameMenu(GAMEOPTIONS_IN);      
    }
    break;
  }

  SetMouseCursor(0,false);   
}

private function ChangeState_SettingsMenu(newState:int):void
{
  switch(newState)
  {
  case SETTINGSOPTIONS_IN: 
    PlaySound(OPTIONSIN_SOUND);  
    break;
  case SETTINGSOPTIONS_OUT:
    PlaySound(OPTIONSOUT_SOUND);  
    break;
  }
  
  settingsMenuState = newState;
  settingsMenuTime = 0;
}

[Embed(source="Images/SettingsOptions.png")] private var settingsOptionsImg:Class;

private var settingsOptionsBitmap:BitmapData = null;

private function Draw_SettingsOptions():void
{
  if (settingsOptionsBitmap == null)
  {   
    settingsOptionsBitmap = new BitmapData(260,200,true,0x00000000);
    settingsOptionsBitmap.draw(new settingsOptionsImg()); 
  }   

  for (var i:int = OPTION_SOUNDVOLUME; i <= OPTION_EXIT; i++)
  { 
    var srcY:int = pressedOption == i ? 160 : 80;
         
    var srcX:int = 0;
    
    if (settingsMenuState == SETTINGSOPTIONS_IN) 
    {
      srcX = -((1 - settingsMenuTime) * 3 - Number((OPTION_SETTINGS - i) / 2)) * 400;
      
      if (srcX > 0)
        srcX = 0;      
    }
      
    if (settingsMenuState == SETTINGSOPTIONS_OUT)
    {
      srcX = (settingsMenuTime * 3 - Number(i / 2)) * 400;
      
      if (srcX < 0)
        srcX = 0;   
    }  
  
    screenBuffer.copyPixels(gameOptionBackgroundBitmap,
                            new Rectangle(0, srcY, 340, 80),
                            new Point(70 + srcX,225 + i * 70 + (pressedOption == i ? 1 : 0)));
    
    switch (i)
    {
    case OPTION_SOUNDVOLUME:
      srcY = 0;
      break;
    case OPTION_MUSICVOLUME:
      srcY = 50;
      break;
    case OPTION_EXIT:
      srcY = 100;
      break;
    }
    
    screenBuffer.copyPixels(settingsOptionsBitmap,
                            new Rectangle(0, srcY, 260, 50),
                            new Point(110 + srcX,240 + i * 70 + (pressedOption == i ? 1 : 0)));
                     

    switch (i)
    {
    case OPTION_SOUNDVOLUME:
      screenBuffer.copyPixels(settingsOptionsBitmap,
                              new Rectangle(0, 150, 260, 25),
                              new Point(110 + srcX,267 + i * 70));  
                              
      screenBuffer.copyPixels(settingsOptionsBitmap,
                              new Rectangle(24, 183, int(212 * soundVolume), 7),
                              new Point(134 + srcX,275 + i * 70));                               
      break;
    case OPTION_MUSICVOLUME:
      screenBuffer.copyPixels(settingsOptionsBitmap,
                              new Rectangle(0, 150, 260, 25),
                              new Point(110 + srcX,267 + i * 70)); 
                              
      screenBuffer.copyPixels(settingsOptionsBitmap,
                              new Rectangle(24, 183, int(212 * musicVolume), 7),
                              new Point(134 + srcX,275 + i * 70));                                 
      break;
    }                          
  }  
}

private function MouseDown_SettingsMenu(event:MouseEvent):void 	
{	
	var mx:int = event.localX;
  var my:int = event.localY;
	  
	if (settingsMenuState == SETTINGSOPTIONS_WAIT)
  { 
    if ((mx >= 130) && (mx <= 341) && (my >= 273) && (my <= 283)) 
    {	  	 
      soundVolume = (mx - 130) / 211;
      StoreData(SOUND_VOLUME);
      PlaySound(OPTIONSELECTED_SOUND);          
    }
    
    if ((mx >= 130) && (mx <= 341) && (my >= 343) && (my <= 353)) 
    {	  	 
      musicVolume = (mx - 130) / 211;
      StoreData(MUSIC_VOLUME);      
      PlaySound(OPTIONSELECTED_SOUND);          
    } 
  
    if ((mx >= 110) && (mx < 370) && (my >= 380) && (my < 430)) 
    {	  	 
      pressedOption = OPTION_EXIT;
      PlaySound(OPTIONSELECTED_SOUND);          
    }
  }
}	
	
private function MouseUp_SettingsMenu(event:MouseEvent):void 	
{	
  var mx:int = event.localX;
  var my:int = event.localY;
  
	if (settingsMenuState == SETTINGSOPTIONS_WAIT) 
    switch (pressedOption)
    {
    case OPTION_EXIT:		  
      selectedOption = OPTION_EXIT;      
      ChangeState_SettingsMenu(SETTINGSOPTIONS_OUT);      
      break;
    }
  
  pressedOption = -1;
}

private function MouseMoved_SettingsMenu(event:MouseEvent):void 	
{
  var mx:int = event.localX;
  var my:int = event.localY;
  
  for (var i:int = OPTION_SOUNDVOLUME; i <= OPTION_EXIT; i++)
    if (pressedOption == i)
      if ((mx < 110) || (mx > 370) || (my < 240 + i * 70) || (my > 290 + i * 70)) 	  	 
        pressedOption = -1; 
}