import flash.display.*; 
import flash.events.*;
import flash.external.ExternalInterface; 
import mx.events.*;
import mx.controls.*;
import mx.managers.CursorManager;
import Classes.*;

include "Sounds.as";
include "Effects.as";

include "Activation.as";
include "GameMenu.as";
include "SettingsMenu.as";
include "Game.as";

private static const SCREEN_WIDTH:int     = 480;
private static const SCREEN_HEIGHT:int    = 480;

private static const ACTIVATION:int       = 0;
private static const GAME_MENU:int        = 1;
private static const SETTINGS_MENU:int    = 2;
private static const GAME:int             = 3;

private var state:int = GAME_MENU;

private var active:Boolean = true;
private var initializationCompleted:Boolean = false;
private var screenBuffer:BitmapData = null;

public function Init():void
{  	
  SetMouseCursor(0,false);

  screenBuffer = new BitmapData(SCREEN_WIDTH, SCREEN_HEIGHT, false, 0x00000000);
  
	if (ExternalInterface.available)
    state = ACTIVATION;   
    
  LoadData();
  
  //state = GAME;
  //New_Game();
  
  initializationCompleted = true; 
}

private var prevTime:Date = null;

private function UpdateFrame():void
{
  if (!initializationCompleted)
    return;

  var currTime:Date = new Date();
  var ellapsedTime:Number = 0;
  
  if (prevTime != null) 
    ellapsedTime = (currTime.getTime() - prevTime.getTime()) / 1000.0;
    
  prevTime = currTime;  
  
  Draw(ellapsedTime);
    
  gamePanel.graphics.clear();    
  gamePanel.graphics.beginBitmapFill(screenBuffer, null, false, false);
  gamePanel.graphics.drawRect(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
  gamePanel.graphics.endFill();  
   
  /*gamePanel.graphics.beginFill(0xFF0000,0.5);
  gamePanel.graphics.drawRect(snake.getBounds(0).x, SCREEN_HEIGHT - (snake.getBounds(0).y - scrollPos), snake.getBounds(0).width, snake.getBounds(0).height);
  gamePanel.graphics.endFill();*/
  
  gamePanel.setFocus(); 
}


private function Activated(event:Event):void
{
  gamePanel.setFocus(); 
  active = true;     
}

private function Deactivated(event:Event):void
{
  active = false; 
    
  if ((state == GAME) && (gameState == GAME_PLAYED))
  {
    saveState = gameState;
    ChangeState_Game(GAME_DEACTIVATED);   
  }
}

private function Draw(ellapsedTime:Number):void
{
  switch (state)
  {
  case ACTIVATION:
    Draw_Activation(ellapsedTime);
    break;  
  case GAME_MENU:
    Draw_GameMenu(ellapsedTime);
    break;
  case SETTINGS_MENU:
    Draw_SettingsMenu(ellapsedTime);
    break;    
  case GAME:
    Draw_Game(ellapsedTime);  
    break;   
  }
}

[Embed(source="Images/MouseCursor.png")]private var mouseCursorImg:Class;
[Embed(source="Images/MouseCursor_N.png")]private var mouseCursorNImg:Class;
[Embed(source="Images/MouseCursor_ND.png")]private var mouseCursorNDImg:Class;
[Embed(source="Images/MouseCursor_E.png")]private var mouseCursorEImg:Class;
[Embed(source="Images/MouseCursor_ED.png")]private var mouseCursorEDImg:Class;
[Embed(source="Images/MouseCursor_S.png")]private var mouseCursorSImg:Class;
[Embed(source="Images/MouseCursor_SD.png")]private var mouseCursorSDImg:Class;
[Embed(source="Images/MouseCursor_W.png")]private var mouseCursorWImg:Class;
[Embed(source="Images/MouseCursor_WD.png")]private var mouseCursorWDImg:Class;
[Embed(source="Images/MouseCursor_Empty.png")]private var mouseCursorEmptyImg:Class;

private static const EMPTY_CURSOR:int     = 10;
private var cursorID:Number = 0;
private var mouseCursorDirection:int = -1;
private var mouseCursorDisabled:Boolean = false;

private function SetMouseCursor(direction:int,disabled:Boolean):void 	
{
  if ((mouseCursorDirection != direction) || (mouseCursorDisabled != disabled))
  {
    mouseCursorDirection = direction;
    mouseCursorDisabled = disabled;
  
		CursorManager.removeCursor(cursorID);	  
	  
	  switch (direction)
	  {
	  case 0:
      cursorID = CursorManager.setCursor(mouseCursorImg);
      break;
	  case Snake.NORTH:
      if (disabled)
        cursorID = CursorManager.setCursor(mouseCursorNDImg);
      else
        cursorID = CursorManager.setCursor(mouseCursorNImg);      
      break; 
	  case Snake.EAST:
      if (disabled)
        cursorID = CursorManager.setCursor(mouseCursorEDImg);
      else
        cursorID = CursorManager.setCursor(mouseCursorEImg);      
      break; 
	  case Snake.SOUTH:
      if (disabled)
        cursorID = CursorManager.setCursor(mouseCursorSDImg);
      else
        cursorID = CursorManager.setCursor(mouseCursorSImg);      
      break; 
	  case Snake.WEST:
      if (disabled)
        cursorID = CursorManager.setCursor(mouseCursorWDImg);
      else
        cursorID = CursorManager.setCursor(mouseCursorWImg);      
      break;
    case EMPTY_CURSOR:
      cursorID = CursorManager.setCursor(mouseCursorEmptyImg);    
      break;        
    }
	}
}

private function MouseDown(event:MouseEvent):void 	
{	
  switch (state)
  {
  case ACTIVATION:
    MouseDown_Activation(event);
    break;  
  case GAME_MENU:
    MouseDown_GameMenu(event);
    break;
  case SETTINGS_MENU:
    MouseDown_SettingsMenu(event);
    break;    
  case GAME:
    MouseDown_Game(event);
    break;
  }
}	
	
private function MouseUp(event:MouseEvent):void 	
{	
  switch (state)
  {
  case ACTIVATION:
    MouseUp_Activation(event);
    break;  
  case GAME_MENU:
    MouseUp_GameMenu(event);
    break;
  case SETTINGS_MENU:
    MouseUp_SettingsMenu(event);
    break;      
  case GAME:
    MouseUp_Game(event);
    break;
  }
}

private function MouseMoved(event:MouseEvent):void 	
{
  switch (state)
  {
  case GAME_MENU:
    MouseMoved_GameMenu(event);
    break;
  case SETTINGS_MENU:
    MouseMoved_SettingsMenu(event);
    break;    
  case GAME:
    MouseMoved_Game(event);
    break;
  }
}

private var prevDate:Date = null; 
	 
private function MouseWheelMoved(event:MouseEvent):void 
{ 
  if (prevDate == null) 
    prevDate = new Date(); 
  else 
    if (new Date().time - prevDate.time < 50) 
      return; 
    else 
      prevDate = new Date();
      
  switch (state)
  {
  case GAME:
    MouseWheelMoved_Game(event);
    break;
  }
	  
	event.preventDefault(); 
}

private function KeyDown(event:KeyboardEvent):void 
{	  
  switch (state)
  {
  case GAME:
    KeyDown_Game(event);
    break;
  }
}

private function KeyUp(event:KeyboardEvent):void 	
{
  switch (state)
  {  
  case GAME:  
    KeyUp_Game(event);
    break;
  }
}

private function LoadData():void 	
{
	var so:SharedObject = SharedObject.getLocal("SnakesAdventure");
	  
	if (so.size > 0)	  
	{           
    if (so.data.hiscore != null)
      hiscore = so.data.hiscore;
      
    if (so.data.soundVolume != null)
      soundVolume = so.data.soundVolume;
      
    if (so.data.musicVolume != null)
      musicVolume = so.data.musicVolume;      
	}
}

private static const HISCORE:int     = 1;
private static const SOUND_VOLUME:int = 2;
private static const MUSIC_VOLUME:int = 3;
	
private function StoreData(what:int):void 	
{
	var so:SharedObject = SharedObject.getLocal("SnakesAdventure");

  switch (what)
  {
  case HISCORE:
    so.data.hiscore = hiscore;	
    break;
  case SOUND_VOLUME:
    so.data.soundVolume = soundVolume;	
    break;
  case MUSIC_VOLUME:
    so.data.musicVolume = musicVolume;	
    break;
  }
	  
	so.flush();	  
}	
